@@ -2,19 +2,19 @@ from typing import List
 from xml.etree import ElementTree
 from xml.etree.ElementTree import Element
 import zipfile
-import pygdtf.type
+from pygdtf.value import *
 
 
 # Standard predefined colour spaces: R, G, B, W-P
-COLOR_SPACE_SRGB = pygdtf.type.ColorSpaceDefinition(
-    pygdtf.type.ColorCIE(0.6400, 0.3300, 0.2126), pygdtf.type.ColorCIE(0.3000, 0.6000, 0.7152),
-    pygdtf.type.ColorCIE(0.1500, 0.0600, 0.0722), pygdtf.type.ColorCIE(0.3127, 0.3290, 1.0000))
-COLOR_SPACE_PROPHOTO = pygdtf.type.ColorSpaceDefinition(
-    pygdtf.type.ColorCIE(0.7347, 0.2653), pygdtf.type.ColorCIE(0.1596, 0.8404),
-    pygdtf.type.ColorCIE(0.0366, 0.0001), pygdtf.type.ColorCIE(0.3457, 0.3585))
-COLOR_SPACE_ANSI = pygdtf.type.ColorSpaceDefinition(
-    pygdtf.type.ColorCIE(0.7347, 0.2653), pygdtf.type.ColorCIE(0.1596, 0.8404),
-    pygdtf.type.ColorCIE(0.0366, 0.001), pygdtf.type.ColorCIE(0.4254, 0.4044))
+COLOR_SPACE_SRGB = ColorSpaceDefinition(
+    ColorCIE(0.6400, 0.3300, 0.2126), ColorCIE(0.3000, 0.6000, 0.7152),
+    ColorCIE(0.1500, 0.0600, 0.0722), ColorCIE(0.3127, 0.3290, 1.0000))
+COLOR_SPACE_PROPHOTO = ColorSpaceDefinition(
+    ColorCIE(0.7347, 0.2653), ColorCIE(0.1596, 0.8404),
+    ColorCIE(0.0366, 0.0001), ColorCIE(0.3457, 0.3585))
+COLOR_SPACE_ANSI = ColorSpaceDefinition(
+    ColorCIE(0.7347, 0.2653), ColorCIE(0.1596, 0.8404),
+    ColorCIE(0.0366, 0.001), ColorCIE(0.4254, 0.4044))
 
 
 def _find_root(pkg: 'zipfile.ZipFile') -> 'ElementTree.Element':
@@ -50,53 +50,82 @@ class FixtureType:
         self.fixture_type_id = self._root.get('FixtureTypeID')
         self.thumbnail = self._root.get('Thumbnail')
         self.ref_ft = self._root.get('RefFT')
-        xactivation_groups = self._root.find('AttributeDefinitions').find('ActivationGroups').findall('ActivationGroup')
-        self.activation_groups = [ActivationGroup(xml_node=i) for i in xactivation_groups]
-        xfeature_groups = self._root.find('AttributeDefinitions').find('FeatureGroups').findall('FeatureGroup')
-        self.feature_groups = [FeatureGroup(xml_node=i) for i in xfeature_groups]
-        xattributes = self._root.find('AttributeDefinitions').find('Attributes').findall('Attribute')
-        self.attributes = [Attribute(xml_node=i) for i in xattributes]
-        xwheels = self._root.find('Wheels').findall('Wheel')
-        self.wheels = [Wheel(xml_node=i) for i in xwheels]
-        xemitters = self._root.find('PhysicalDescriptions').find('Emitters').findall('Emitter')
-        self.emitters = [Emitter(xml_node=i) for i in xemitters]
-        xfilters = self._root.find('PhysicalDescriptions').find('Filters').findall('Filter')
-        self.filters = [Filter(xml_node=i) for i in xfilters]
-        self.color_space = self._root.find('PhysicalDescriptions').find('ColorSpace')
-        xdmx_profiles = self._root.find('PhysicalDescriptions').find('DMXProfiles').findall('DMXProfile')
-        self.dmx_profiles = [DmxProfile(xml_node=i) for i in xdmx_profiles]
-        xcri_groups = self._root.find('PhysicalDescriptions').find('CRIs').findall('CRIGroup')
-        self.cri_groups = [CriGroup(xml_node=i) for i in xcri_groups]
-        xmodels = self._root.find('Models').findall('Model')
-        self.models = [Model(xml_node=i) for i in xmodels]
-        xgeometry_collect = self._root.find('Geometries')
+        # For each attribute, we first check for the existence of the collect node
+        # If such a node doesn't exist, then none of the children will exist and
+        # the corresponding attribute for this class can be set to empty. Failing
+        # to do this would result in AttributeError if we try to, for example, run
+        # a findall on a non-existent collect
+        if activation_collect := self._root.find('AttributeDefinitions').find('ActivationGroups'):
+            self.activation_groups = [ActivationGroup(xml_node=i) for i in activation_collect.findall('ActivationGroup')]
+        else:
+            self.activation_groups = []
+        if feature_collect := self._root.find('AttributeDefinitions').find('FeatureGroups'):
+            self.feature_groups = [FeatureGroup(xml_node=i) for i in feature_collect.findall('FeatureGroup')]
+        else:
+            self.feature_groups = []
+        if attribute_collect := self._root.find('AttributeDefinitions').find('Attributes'):
+            self.attributes = [Attribute(xml_node=i) for i in attribute_collect.findall('Attribute')]
+        else:
+            self.attributes = []
+        if wheel_collect := self._root.find('Wheels'):
+            self.wheels = [Wheel(xml_node=i) for i in wheel_collect.findall('Wheel')]
+        else:
+            self.wheels = []
+        if emitter_collect := self._root.find('PhysicalDescriptions').find('Emitters'):
+            self.emitters = [Emitter(xml_node=i) for i in emitter_collect.findall('Emitter')]
+        else:
+            self.emitters = []
+        if filter_collect := self._root.find('PhysicalDescriptions').find('Filters'):
+            self.filters = [Filter(xml_node=i) for i in filter_collect.findall('Filter')]
+        else:
+            self.filters = []
+        if color_space := self._root.find('PhysicalDescriptions').find('ColorSpace'):
+            self.color_space = ColorSpace(xml_node=color_space)
+        else:
+            # The default color space is sRGB if nothing else is defined
+            self.color_space = ColorSpace(mode=ColorSpaceMode('sRGB'))
+        if profiles_collect := self._root.find('PhysicalDescriptions').find('DMXProfiles'):
+            self.dmx_profiles = [DmxProfile(xml_node=i) for i in profiles_collect.findall('DMXProfile')]
+        else:
+            self.dmx_profiles = []
+        if cri_collect := self._root.find('PhysicalDescriptions').find('CRIs'):
+            self.cri_groups = [CriGroup(xml_node=i) for i in cri_collect.findall('CRIGroup')]
+        else:
+            self.cri_groups = []
+        if model_collect := self._root.find('Models'):
+            self.models = [Model(xml_node=i) for i in model_collect.findall('Model')]
         self.geometries = []
-        for i in xgeometry_collect.findall('Geometry'):
-            self.geometries.append(Geometry(xml_node=i))
-        for i in xgeometry_collect.findall('Axis'):
-            self.geometries.append(GeometryAxis(xml_node=i))
-        for i in xgeometry_collect.findall('FilterBeam'):
-            self.geometries.append(GeometryFilterBeam(xml_node=i))
-        for i in xgeometry_collect.findall('FilterColor'):
-            self.geometries.append(GeometryFilterColor(xml_node=i))
-        for i in xgeometry_collect.findall('FilterGobo'):
-            self.geometries.append(GeometryFilterGobo(xml_node=i))
-        for i in xgeometry_collect.findall('FilterShaper'):
-            self.geometries.append(GeometryFilterShaper(xml_node=i))
-        for i in xgeometry_collect.findall('Beam'):
-            self.geometries.append(GeometryBeam(xml_node=i))
-        for i in xgeometry_collect.findall('GeometryReference'):
-            self.geometries.append(GeometryReference(xml_node=i))
-        xdmx_modes = self._root.find('DMXModes').findall('DMXMode')
-        self.dmx_modes = [DmxMode(xml_node=i) for i in xdmx_modes]
-        xrevisions = self._root.find('Revisions').findall('Revision')
-        self.revisions = [Revision(xml_node=i) for i in xrevisions]
+        if geometry_collect := self._root.find('Geometries'):
+            for i in geometry_collect.findall('Geometry'):
+                self.geometries.append(Geometry(xml_node=i))
+            for i in geometry_collect.findall('Axis'):
+                self.geometries.append(GeometryAxis(xml_node=i))
+            for i in geometry_collect.findall('FilterBeam'):
+                self.geometries.append(GeometryFilterBeam(xml_node=i))
+            for i in geometry_collect.findall('FilterColor'):
+                self.geometries.append(GeometryFilterColor(xml_node=i))
+            for i in geometry_collect.findall('FilterGobo'):
+                self.geometries.append(GeometryFilterGobo(xml_node=i))
+            for i in geometry_collect.findall('FilterShaper'):
+                self.geometries.append(GeometryFilterShaper(xml_node=i))
+            for i in geometry_collect.findall('Beam'):
+                self.geometries.append(GeometryBeam(xml_node=i))
+            for i in geometry_collect.findall('GeometryReference'):
+                self.geometries.append(GeometryReference(xml_node=i))
+        if dmx_mode_collect := self._root.find('DMXModes'):
+            self.dmx_modes = [DmxMode(xml_node=i) for i in dmx_mode_collect.findall('DMXMode')]
+        else:
+            self.dmx_modes = []
+        if revision_collect := self._root.find('Revisions'):
+            self.revisions = [Revision(xml_node=i) for i in revision_collect.findall('Revision')]
+        else:
+            self.revisions = []
 
-    def get_geometry_by_type(self, geometry_type):
+    def get_geometry_by_type(self, geometry_class):
         """Recursively find all geometries of a given type"""
         def iterate_geometries(collector):
             for g in collector.geometries:
-                if g.geometry_type == geometry_type:
+                if type(g) == geometry_class:
                     matched.append(g)
                 iterate_geometries(g)
         matched = []
@@ -121,7 +150,7 @@ class ActivationGroup(BaseNode):
         super().__init__(*args, **kwargs)
 
     def _read_xml(self, xml_node: 'Element'):
-        self.name = xml_node.get('Name')
+        self.name = xml_node.attrib.get('Name')
 
 
 class FeatureGroup(BaseNode):
@@ -137,8 +166,8 @@ class FeatureGroup(BaseNode):
         super().__init__(*args, **kwargs)
 
     def _read_xml(self, xml_node: 'Element'):
-        self.name = xml_node.get('Name')
-        self.pretty = xml_node.get('Pretty')
+        self.name = xml_node.attrib.get('Name')
+        self.pretty = xml_node.attrib.get('Pretty')
         self.features = [Feature(xml_node=i) for i in xml_node.findall('Feature')]
 
 
@@ -149,14 +178,14 @@ class Feature(BaseNode):
         super().__init__(*args, **kwargs)
 
     def _read_xml(self, xml_node: 'Element'):
-        self.name = xml_node.get('Name')
+        self.name = xml_node.attrib.get('Name')
 
 
 class Attribute(BaseNode):
 
     def __init__(self, name: str = None, pretty: str = None,
                  activation_group: 'NodeLink' = None, feature: 'NodeLink' = None,
-                 main_attribute: 'NodeLink' = None, physical_unit: 'PhysicalUnit' = pygdtf.type.PhysicalUnit(None),
+                 main_attribute: 'NodeLink' = None, physical_unit: 'PhysicalUnit' = PhysicalUnit(None),
                  color: 'ColorCIE' = None, *args, **kwargs):
         self.name = name
         self.pretty = pretty
@@ -168,13 +197,13 @@ class Attribute(BaseNode):
         super().__init__(*args, **kwargs)
 
     def _read_xml(self, xml_node: 'Element'):
-        self.name = xml_node.get('Name')
-        self.pretty = xml_node.get('Pretty')
-        self.activation_group = pygdtf.type.NodeLink('ActivationGroups', xml_node.get('ActivationGroup'))
-        self.feature = pygdtf.type.NodeLink('FeatureGroups', xml_node.get('Feature'))
-        self.main_attribute = pygdtf.type.NodeLink('Attribute', xml_node.get('MainAttribute'))
-        self.physical_unit = pygdtf.type.PhysicalUnit(xml_node.get('PhysicalUnit'))
-        self.color = pygdtf.type.ColorCIE(str_repr=xml_node.get('Color'))
+        self.name = xml_node.attrib.get('Name')
+        self.pretty = xml_node.attrib.get('Pretty')
+        self.activation_group = NodeLink('ActivationGroups', xml_node.attrib.get('ActivationGroup'))
+        self.feature = NodeLink('FeatureGroups', xml_node.attrib.get('Feature'))
+        self.main_attribute = NodeLink('Attribute', xml_node.attrib.get('MainAttribute'))
+        self.physical_unit = PhysicalUnit(xml_node.attrib.get('PhysicalUnit'))
+        self.color = ColorCIE(str_repr=xml_node.attrib.get('Color'))
 
 
 class Wheel(BaseNode):
@@ -188,18 +217,18 @@ class Wheel(BaseNode):
         super().__init__(*args, **kwargs)
 
     def _read_xml(self, xml_node: 'Element'):
-        self.name = xml_node.get('Name')
+        self.name = xml_node.attrib.get('Name')
         self.wheel_slots = [WheelSlot(xml_node=i) for i in xml_node.findall('Slot')]
 
 
 class WheelSlot(BaseNode):
 
     def __init__(self, name: str = None, color: 'ColorCIE' = None,
-                 filter: 'NodeLink' = None, media_file_name: 'Resource' = None,
+                 whl_filter: 'NodeLink' = None, media_file_name: 'Resource' = None,
                  facets: List['PrismFacet'] = None, *args, **kwargs):
         self.name = name
         self.color = color
-        self.filter = filter
+        self.filter = whl_filter
         self.media_file_name = media_file_name
         if facets is not None:
             self.facets = facets
@@ -208,10 +237,10 @@ class WheelSlot(BaseNode):
         super().__init__(*args, **kwargs)
 
     def _read_xml(self, xml_node: 'Element'):
-        self.name = xml_node.get('Name')
-        self.color = pygdtf.type.ColorCIE(str_repr=xml_node.get('Color'))
-        self.filter = pygdtf.type.NodeLink('FilterCollect', xml_node.get('Filter'))
-        self.media_file_name = pygdtf.type.Resource(xml_node.get('MediaFileName'), 'png')
+        self.name = xml_node.attrib.get('Name')
+        self.color = ColorCIE(str_repr=xml_node.attrib.get('Color'))
+        self.filter = NodeLink('FilterCollect', xml_node.attrib.get('Filter'))
+        self.media_file_name = Resource(xml_node.attrib.get('MediaFileName'), 'png')
         self.facets = [PrismFacet(xml_node=i) for i in xml_node.findall('Facet')]
 
 
@@ -223,8 +252,8 @@ class PrismFacet(BaseNode):
         super().__init__(*args, **kwargs)
 
     def _read_xml(self, xml_node: 'Element'):
-        self.color = pygdtf.type.ColorCIE(str_repr=xml_node.get('Color'))
-        self.rotation = pygdtf.type.Rotation(str_repr=xml_node.get('Rotation'))
+        self.color = ColorCIE(str_repr=xml_node.attrib.get('Color'))
+        self.rotation = Rotation(str_repr=xml_node.attrib.get('Rotation'))
 
 
 class Emitter(BaseNode):
@@ -238,13 +267,13 @@ class Emitter(BaseNode):
         super().__init__(*args, **kwargs)
 
     def _read_xml(self, xml_node: 'Element'):
-        self.name = xml_node.get('Name')
-        self.color = pygdtf.type.ColorCIE(str_repr=xml_node.get('Color'))
+        self.name = xml_node.attrib.get('Name')
+        self.color = ColorCIE(str_repr=xml_node.attrib.get('Color'))
         try:
-            self.dominant_wave_length = float(xml_node.get('DominantWaveLength'))
+            self.dominant_wave_length = float(xml_node.attrib.get('DominantWaveLength'))
         except TypeError:
             self.dominant_wave_length = None
-        self.diode_part = xml_node.get('DiodePart')
+        self.diode_part = xml_node.attrib.get('DiodePart')
         self.measurements = [Measurement(xml_node=i) for i in xml_node.findall('Measurement')]
 
 
@@ -261,15 +290,15 @@ class Filter(BaseNode):
         super().__init__(*args, **kwargs)
 
     def _read_xml(self, xml_node: 'Element'):
-        self.name = xml_node.get('Name')
-        self.color = pygdtf.type.ColorCIE(str_repr=xml_node.get('Color'))
+        self.name = xml_node.attrib.get('Name')
+        self.color = ColorCIE(str_repr=xml_node.attrib.get('Color'))
         self.measurements = [Measurement(xml_node=i) for i in xml_node.findall('Measurement')]
 
 
 class Measurement(BaseNode):
 
     def __init__(self, physical: float = None, luminous_intensity: float = None,
-                 transmission: float = None, interpolation_to: 'InterpolationTo' = pygdtf.type.InterpolationTo(None),
+                 transmission: float = None, interpolation_to: 'InterpolationTo' = InterpolationTo(None),
                  *args, **kwargs):
         self.physical = physical
         self.luminous_intensity = luminous_intensity
@@ -278,16 +307,16 @@ class Measurement(BaseNode):
         super().__init__(*args, **kwargs)
 
     def _read_xml(self, xml_node: 'Element'):
-        self.physical = float(xml_node.get('Physical'))
+        self.physical = float(xml_node.attrib.get('Physical'))
         try:
-            self.luminous_intensity = float(xml_node.get('LuminousIntensity'))
+            self.luminous_intensity = float(xml_node.attrib.get('LuminousIntensity'))
         except TypeError:
             self.luminous_intensity = None
         try:
-            self.transmission = float(xml_node.get('Transmission'))
+            self.transmission = float(xml_node.attrib.get('Transmission'))
         except TypeError:
             self.transmission = None
-        self.interpolation_to = pygdtf.type.InterpolationTo(xml_node.get('InterpolationTo'))
+        self.interpolation_to = InterpolationTo(xml_node.attrib.get('InterpolationTo'))
         self.measurement_points = [MeasurementPoint(xml_node=i) for i in xml_node.findall('MeasurementPoint')]
 
 
@@ -299,13 +328,13 @@ class MeasurementPoint(BaseNode):
         super().__init__(*args, **kwargs)
 
     def _read_xml(self, xml_node: 'Element'):
-        self.wave_length = float(xml_node.get('WaveLength'))
-        self.energy = float(xml_node.get('Energy'))
+        self.wave_length = float(xml_node.attrib.get('WaveLength'))
+        self.energy = float(xml_node.attrib.get('Energy'))
 
 
 class ColorSpace(BaseNode):
 
-    def __init__(self, mode: 'ColorSpaceMode' = pygdtf.type.ColorSpaceMode(None),
+    def __init__(self, mode: 'ColorSpaceMode' = ColorSpaceMode(None),
                  definition: 'ColorSpaceDefinition' = None, *args, **kwargs):
         self.mode = mode
         if definition is not None:
@@ -315,12 +344,12 @@ class ColorSpace(BaseNode):
         super().__init__(*args, **kwargs)
 
     def _read_xml(self, xml_node: 'Element'):
-        self.mode = pygdtf.type.ColorSpaceMode(xml_node.get('Mode'))
+        self.mode = ColorSpaceMode(xml_node.attrib.get('Mode'))
         if str(self.mode) == 'Custom':
-            self.red = pygdtf.type.ColorCIE(str_repr=xml_node.get('Red'))
-            self.green = pygdtf.type.ColorCIE(str_repr=xml_node.get('Green'))
-            self.blue = pygdtf.type.ColorCIE(str_repr=xml_node.get('Blue'))
-            self.white_point = pygdtf.type.ColorCIE(str_repr=xml_node.get('WhitePoint'))
+            self.red = ColorCIE(str_repr=xml_node.attrib.get('Red'))
+            self.green = ColorCIE(str_repr=xml_node.attrib.get('Green'))
+            self.blue = ColorCIE(str_repr=xml_node.attrib.get('Blue'))
+            self.white_point = ColorCIE(str_repr=xml_node.attrib.get('WhitePoint'))
         else:
             self._match_definition()
 
@@ -352,27 +381,27 @@ class CriGroup(BaseNode):
         super().__init__(*args, **kwargs)
 
     def _read_xml(self, xml_node: 'Element'):
-        self.color_temperature = float(xml_node.get('ColorTemperature', default=6000))
+        self.color_temperature = float(xml_node.attrib.get('ColorTemperature', 6000))
         self.cris = [Cri(xml_node=i) for i in xml_node.findall('CRI')]
 
 
 class Cri(BaseNode):
 
-    def __init__(self, ces: 'Ces' = pygdtf.type.Ces(None), color_temperature: int = 100, *args, **kwargs):
+    def __init__(self, ces: 'Ces' = Ces(None), color_temperature: int = 100, *args, **kwargs):
         self.ces = ces
         self.color_temperature = color_temperature
         super().__init__(*args, **kwargs)
 
     def _read_xml(self, xml_node: 'Element'):
-        self.ces = pygdtf.type.Ces(xml_node.get('CES'))
-        self.color_temperature = int(xml_node.get('ColorTemperature', default=100))
+        self.ces = Ces(xml_node.attrib.get('CES'))
+        self.color_temperature = int(xml_node.attrib.get('ColorTemperature', 100))
 
 
 class Model(BaseNode):
 
     def __init__(self, name: str = None, length: float = 0, width: float = 0,
-                 height: float = 0, primitive_type: 'PrimitiveType' = pygdtf.type.PrimitiveType(None),
-                 file: pygdtf.type.Resource = None, *args, **kwargs):
+                 height: float = 0, primitive_type: 'PrimitiveType' = PrimitiveType(None),
+                 file: Resource = None, *args, **kwargs):
         self.name = name
         self.length = length
         self.width = width
@@ -382,18 +411,18 @@ class Model(BaseNode):
         super().__init__(*args, **kwargs)
 
     def _read_xml(self, xml_node: 'Element'):
-        self.name = xml_node.get('Name')
-        self.length = float(xml_node.get('Length', default=0))
-        self.width = float(xml_node.get('Width', default=0))
-        self.height = float(xml_node.get('Height', default=0))
-        self.primitive_type = pygdtf.type.PrimitiveType(xml_node.get('PrimitiveType'))
-        self.file = pygdtf.type.Resource(xml_node.get('File'))
+        self.name = xml_node.attrib.get('Name')
+        self.length = float(xml_node.attrib.get('Length', 0))
+        self.width = float(xml_node.attrib.get('Width', 0))
+        self.height = float(xml_node.attrib.get('Height', 0))
+        self.primitive_type = PrimitiveType(xml_node.attrib.get('PrimitiveType'))
+        self.file = Resource(xml_node.attrib.get('File'))
 
 
 class Geometry(BaseNode):
 
     def __init__(self, name: str = None, model: str = None,
-                 position: 'Matrix' = pygdtf.type.Matrix(0), geometries: List = None, *args, **kwargs):
+                 position: 'Matrix' = Matrix(0), geometries: List = None, *args, **kwargs):
         self.name = name
         self.model = model
         self.position = position
@@ -404,9 +433,9 @@ class Geometry(BaseNode):
         super().__init__(*args, **kwargs)
 
     def _read_xml(self, xml_node: 'Element'):
-        self.name = xml_node.get('Name')
-        self.model = xml_node.get('Model')
-        self.position = pygdtf.type.Matrix(xml_node.get('Position', default=0))
+        self.name = xml_node.attrib.get('Name')
+        self.model = xml_node.attrib.get('Model')
+        self.position = Matrix(xml_node.attrib.get('Position', 0))
         for i in xml_node.findall('Geometry'):
             self.geometries.append(Geometry(xml_node=i))
         for i in xml_node.findall('Axis'):
@@ -447,10 +476,10 @@ class GeometryFilterShaper(Geometry):
 
 class GeometryBeam(Geometry):
 
-    def __init__(self, lamp_type: 'LampType' = pygdtf.type.LampType(None), power_consumption: float = 1000,
+    def __init__(self, lamp_type: 'LampType' = LampType(None), power_consumption: float = 1000,
                  luminous_flux: float = 10000, color_temperature: float = 6000,
                  beam_angle: float = 25.0, field_angle: float = 25.0,
-                 beam_radius: float = 0.05, beam_type: pygdtf.type.BeamType = pygdtf.type.BeamType(None),
+                 beam_radius: float = 0.05, beam_type: BeamType = BeamType(None),
                  color_rendering_index: int = 100, *args, **kwargs):
         self.lamp_type = lamp_type
         self.power_consumption = power_consumption
@@ -465,20 +494,20 @@ class GeometryBeam(Geometry):
 
     def _read_xml(self, xml_node: 'Element'):
         super()._read_xml(xml_node)
-        self.lamp_type = pygdtf.type.LampType(xml_node.get('LampType'))
-        self.power_consumption = float(xml_node.get('PowerConsumption', default=1000))
-        self.luminous_flux = float(xml_node.get('LuminousFlux', default=10000))
-        self.color_temperature = float(xml_node.get('ColorTemperature', default=6000))
-        self.beam_angle = float(xml_node.get('BeamAngle', default=25))
-        self.field_angle = float(xml_node.get('FieldAngle', default=25))
-        self.beam_radius = float(xml_node.get('BeamRadius', default=0.05))
-        self.beam_type = pygdtf.type.BeamType(xml_node.get('BeamType'))
-        self.color_rendering_index = int(xml_node.get('ColorRenderingIndex', default=100))
+        self.lamp_type = LampType(xml_node.attrib.get('LampType'))
+        self.power_consumption = float(xml_node.attrib.get('PowerConsumption', 1000))
+        self.luminous_flux = float(xml_node.attrib.get('LuminousFlux', 10000))
+        self.color_temperature = float(xml_node.attrib.get('ColorTemperature', 6000))
+        self.beam_angle = float(xml_node.attrib.get('BeamAngle', 25))
+        self.field_angle = float(xml_node.attrib.get('FieldAngle', 25))
+        self.beam_radius = float(xml_node.attrib.get('BeamRadius', 0.05))
+        self.beam_type = BeamType(xml_node.attrib.get('BeamType'))
+        self.color_rendering_index = int(xml_node.attrib.get('ColorRenderingIndex', 100))
 
 
 class GeometryReference(BaseNode):
 
-    def __init__(self, name: str = None, position: 'Matrix' = pygdtf.type.Matrix(0),
+    def __init__(self, name: str = None, position: 'Matrix' = Matrix(0),
                  geometry: str = None, model: str = None, *args, **kwargs):
         self.name = name
         self.position = position
@@ -487,24 +516,24 @@ class GeometryReference(BaseNode):
         super().__init__(*args, **kwargs)
 
     def _read_xml(self, xml_node: 'Element'):
-        self.name = xml_node.get('Name')
-        self.position = pygdtf.type.Matrix(xml_node.get('Position', default=0))
-        self.geometry = xml_node.get('Geometry')
-        self.model = xml_node.get('Model')
+        self.name = xml_node.attrib.get('Name')
+        self.position = Matrix(xml_node.attrib.get('Position', 0))
+        self.geometry = xml_node.attrib.get('Geometry')
+        self.model = xml_node.attrib.get('Model')
         self.breaks = [Break(xml_node=i) for i in xml_node.findall('Break')]
 
 
 class Break(BaseNode):
 
-    def __init__(self, dmx_offset: 'DmxAddress' = pygdtf.type.DmxAddress('1'),
+    def __init__(self, dmx_offset: 'DmxAddress' = DmxAddress('1'),
                  dmx_break: int = 1, *args, **kwargs):
         self.dmx_offset = dmx_offset
         self.dmx_break = dmx_break
         super().__init__(*args, **kwargs)
 
     def _read_xml(self, xml_node: 'Element'):
-        self.dmx_offset = pygdtf.type.DmxAddress(xml_node.get('DMXOffset'))
-        self.dmx_break = int(xml_node.get('DMXBreak', default=1))
+        self.dmx_offset = DmxAddress(xml_node.attrib.get('DMXOffset'))
+        self.dmx_break = int(xml_node.attrib.get('DMXBreak', 1))
 
 
 class DmxMode(BaseNode):
@@ -530,8 +559,8 @@ class DmxMode(BaseNode):
         super().__init__(*args, **kwargs)
 
     def _read_xml(self, xml_node: 'Element'):
-        self.name = xml_node.get('Name')
-        self.geometry = xml_node.get('Geometry')
+        self.name = xml_node.attrib.get('Name')
+        self.geometry = xml_node.attrib.get('Geometry')
         self.dmx_channels = [DmxChannel(xml_node=i) for i in xml_node.find('DMXChannels').findall('DMXChannel')]
         self.relations = [Relation(xml_node=i) for i in xml_node.find('Relations').findall('Relation')]
         try:
@@ -543,7 +572,7 @@ class DmxMode(BaseNode):
 class DmxChannel(BaseNode):
 
     def __init__(self, dmx_break: int = 1, offset: List[int] = 'None',
-                 default: 'DmxValue' = pygdtf.type.DmxValue('0/1'), highlight: 'DmxValue' = 'None',
+                 default: 'DmxValue' = DmxValue('0/1'), highlight: 'DmxValue' = 'None',
                  geometry: str = None,
                  logical_channels: List['LogicalChannel'] = None, *args, **kwargs):
         self.dmx_break = dmx_break
@@ -556,23 +585,23 @@ class DmxChannel(BaseNode):
 
     def _read_xml(self, xml_node: 'Element'):
         try:
-            self.dmx_break = int(xml_node.get('DMXBreak'))
+            self.dmx_break = int(xml_node.attrib.get('DMXBreak', 1))
         except ValueError:
             self.dmx_break = 'Overwrite'
         try:
-            self.offset = [int(i) for i in xml_node.get('Offset').split(',')]
+            self.offset = [int(i) for i in xml_node.attrib.get('Offset').split(',')]
         except ValueError:
             self.offset = 'None'
-        self.default = pygdtf.type.DmxValue(xml_node.get('Default', default='0/1'))
-        self.highlight = pygdtf.type.DmxValue(xml_node.get('Highlight'))
-        self.geometry = xml_node.get('Geometry')
+        self.default = DmxValue(xml_node.attrib.get('Default', '0/1'))
+        self.highlight = DmxValue(xml_node.attrib.get('Highlight'))
+        self.geometry = xml_node.attrib.get('Geometry')
         self.logical_channels = [LogicalChannel(xml_node=i) for i in xml_node.findall('LogicalChannel')]
 
 
 class LogicalChannel(BaseNode):
 
-    def __init__(self, attribute: 'NodeLink' = None, snap: 'Snap' = pygdtf.type.Snap(None),
-                 master: 'Master' = pygdtf.type.Master(None), mib_fade: float = 0,
+    def __init__(self, attribute: 'NodeLink' = None, snap: 'Snap' = Snap(None),
+                 master: 'Master' = Master(None), mib_fade: float = 0,
                  dmx_change_time_limit: float = 0,
                  channel_functions: List['ChannelFunction'] = None, *args, **kwargs):
         self.attribute = attribute
@@ -587,22 +616,22 @@ class LogicalChannel(BaseNode):
         super().__init__(*args, **kwargs)
 
     def _read_xml(self, xml_node: 'Element'):
-        self.attribute = pygdtf.type.NodeLink('Attributes', xml_node.get('Attribute'))
-        self.snap = pygdtf.type.Snap(xml_node.get('Snap'))
-        self.master = pygdtf.type.Master(xml_node.get('Master'))
-        self.mib_fade = float(xml_node.get('MibFade', default=0))
-        self.dmx_change_time_limit = float(xml_node.get('DMXChangeTimeLimit', default=0))
+        self.attribute = NodeLink('Attributes', xml_node.attrib.get('Attribute'))
+        self.snap = Snap(xml_node.attrib.get('Snap'))
+        self.master = Master(xml_node.attrib.get('Master'))
+        self.mib_fade = float(xml_node.attrib.get('MibFade', 0))
+        self.dmx_change_time_limit = float(xml_node.attrib.get('DMXChangeTimeLimit', 0))
         self.channel_functions = [ChannelFunction(xml_node=i) for i in xml_node.findall('ChannelFunction')]
 
 
 class ChannelFunction(BaseNode):
 
     def __init__(self, name: str = None, attribute: 'NodeLink' = 'NoFeature',
-                 original_attribute: str = None, dmx_from: 'DmxValue' = pygdtf.type.DmxValue('0/1'),
+                 original_attribute: str = None, dmx_from: 'DmxValue' = DmxValue('0/1'),
                  physical_from: float = 0, physical_to: float = 1, real_fade: float = 0,
-                 wheel: 'NodeLink' = None, emitter: 'NodeLink' = None, filter: 'NodeLink' = None,
-                 dmx_invert: 'DmxInvert' = pygdtf.type.DmxInvert(None), mode_master: 'NodeLink' = None,
-                 mode_from: 'DmxValue' = pygdtf.type.DmxValue('0/1'), mode_to: 'DmxValue' = pygdtf.type.DmxValue('0/1'),
+                 wheel: 'NodeLink' = None, emitter: 'NodeLink' = None, chn_filter: 'NodeLink' = None,
+                 dmx_invert: 'DmxInvert' = DmxInvert(None), mode_master: 'NodeLink' = None,
+                 mode_from: 'DmxValue' = DmxValue('0/1'), mode_to: 'DmxValue' = DmxValue('0/1'),
                  channel_sets: List['ChannelSet'] = None, *args, **kwargs):
         self.name = name
         self.attribute = attribute
@@ -613,7 +642,7 @@ class ChannelFunction(BaseNode):
         self.real_fade = real_fade
         self.wheel = wheel
         self.emitter = emitter
-        self.filter = filter
+        self.filter = chn_filter
         self.dmx_invert = dmx_invert
         self.mode_master = mode_master
         self.mode_from = mode_from
@@ -625,26 +654,26 @@ class ChannelFunction(BaseNode):
         super().__init__(*args, **kwargs)
 
     def _read_xml(self, xml_node: 'Element'):
-        self.name = xml_node.get('Name')
-        self.attribute = pygdtf.type.NodeLink('Attributes', xml_node.get('Attribute', default='NoFeature'))
-        self.original_attribute = xml_node.get('OriginalAttribute')
-        self.dmx_from = pygdtf.type.DmxValue(xml_node.get('DMXFrom', default='0/1'))
-        self.physical_from = float(xml_node.get('PhysicalFrom', default=0))
-        self.physical_to = float(xml_node.get('PhysicalTo', default=1))
-        self.real_fade = float(xml_node.get('RealFade', default=0))
-        self.wheel = pygdtf.type.NodeLink('WheelCollect', xml_node.get('Wheel'))
-        self.emitter = pygdtf.type.NodeLink('EmitterCollect', xml_node.get('Emitter'))
-        self.filter = pygdtf.type.NodeLink('FilterCollect', xml_node.get('Filter'))
-        self.dmx_invert = pygdtf.type.DmxInvert(xml_node.get('DMXInvert'))
-        self.mode_master = pygdtf.type.Master(xml_node.get('ModeMaster'))
-        self.mode_from = pygdtf.type.DmxValue(xml_node.get('ModeFrom', default='0/1'))
-        self.mode_to = pygdtf.type.DmxValue(xml_node.get('ModeTo', default='0/1'))
+        self.name = xml_node.attrib.get('Name')
+        self.attribute = NodeLink('Attributes', xml_node.attrib.get('Attribute', 'NoFeature'))
+        self.original_attribute = xml_node.attrib.get('OriginalAttribute')
+        self.dmx_from = DmxValue(xml_node.attrib.get('DMXFrom', '0/1'))
+        self.physical_from = float(xml_node.attrib.get('PhysicalFrom', 0))
+        self.physical_to = float(xml_node.attrib.get('PhysicalTo', 1))
+        self.real_fade = float(xml_node.attrib.get('RealFade', 0))
+        self.wheel = NodeLink('WheelCollect', xml_node.attrib.get('Wheel'))
+        self.emitter = NodeLink('EmitterCollect', xml_node.attrib.get('Emitter'))
+        self.filter = NodeLink('FilterCollect', xml_node.attrib.get('Filter'))
+        self.dmx_invert = DmxInvert(xml_node.attrib.get('DMXInvert'))
+        self.mode_master = Master(xml_node.attrib.get('ModeMaster'))
+        self.mode_from = DmxValue(xml_node.attrib.get('ModeFrom', '0/1'))
+        self.mode_to = DmxValue(xml_node.attrib.get('ModeTo', '0/1'))
         self.channel_sets = [ChannelSet(xml_node=i) for i in xml_node.findall('ChannelSet')]
 
 
 class ChannelSet(BaseNode):
 
-    def __init__(self, name: str = None, dmx_from: 'DmxValue' = pygdtf.type.DmxValue('0/1'),
+    def __init__(self, name: str = None, dmx_from: 'DmxValue' = DmxValue('0/1'),
                  physical_from: float = 0, physical_to: float = 1,
                  wheel_slot_index: int = 1, *args, **kwargs):
         self.name = name
@@ -655,29 +684,29 @@ class ChannelSet(BaseNode):
         super().__init__(*args, **kwargs)
 
     def _read_xml(self, xml_node: 'Element'):
-        self.name = xml_node.get('Name')
-        self.dmx_from = pygdtf.type.DmxValue(xml_node.get('DMXFrom', default='0/1'))
-        self.physical_from = float(xml_node.get('PhysicalFrom', default=0))
-        self.physical_to = float(xml_node.get('PhysicalTo', default=1))
-        self.wheel_slot_index = int(xml_node.get('WheelSlotIndex', default=1))
+        self.name = xml_node.attrib.get('Name')
+        self.dmx_from = DmxValue(xml_node.attrib.get('DMXFrom', '0/1'))
+        self.physical_from = float(xml_node.attrib.get('PhysicalFrom', 0))
+        self.physical_to = float(xml_node.attrib.get('PhysicalTo', 1))
+        self.wheel_slot_index = int(xml_node.attrib.get('WheelSlotIndex', 1))
 
 
 class Relation(BaseNode):
 
     def __init__(self, name: str = None, master: 'NodeLink' = None,
-                 follower: 'NodeLink' = None, type: 'RelationType' = pygdtf.type.RelationType(None),
+                 follower: 'NodeLink' = None, rel_type: 'RelationType' = RelationType(None),
                  *args, **kwargs):
         self.name = name
         self.master = master
         self.follower = follower
-        self.type = type
+        self.type = rel_type
         super().__init__(*args, **kwargs)
 
     def _read_xml(self, xml_node: 'Element'):
-        self.name = xml_node.get('Name')
-        self.master = pygdtf.type.NodeLink('DMXMode', xml_node.get('Master'))
-        self.follower = pygdtf.type.NodeLink('DMXMode', xml_node.get('Follower'))
-        self.type = pygdtf.type.RelationType(xml_node.get('Type'))
+        self.name = xml_node.attrib.get('Name')
+        self.master = NodeLink('DMXMode', xml_node.attrib.get('Master'))
+        self.follower = NodeLink('DMXMode', xml_node.attrib.get('Follower'))
+        self.type = RelationType(xml_node.attrib.get('Type'))
 
 
 class Macro(BaseNode):
@@ -694,7 +723,7 @@ class Macro(BaseNode):
         super().__init__(*args, **kwargs)
 
     def _read_xml(self, xml_node: 'Element'):
-        self.name = xml_node.get('Name')
+        self.name = xml_node.attrib.get('Name')
         try:
             self.dmx_steps = [MacroDmxStep(xml_node=i) for i in xml_node.find('MacroDMX').findall('MacroDMXStep')]
         except AttributeError:
@@ -717,20 +746,20 @@ class MacroDmxStep(BaseNode):
         super().__init__(*args, **kwargs)
 
     def _read_xml(self, xml_node: 'Element'):
-        self.duration = int(xml_node.get('Duration'))
+        self.duration = int(xml_node.attrib.get('Duration'))
         self.dmx_values = [MacroDmxValue(xml_node=i) for i in xml_node.findall('MacroDMXValue')]
 
 
 class MacroDmxValue(BaseNode):
 
-    def __init__(self, value: 'DmxValue' = None, dmx_channel: 'NodeLink' = None, *args, **kwargs):
-        self.value = value
+    def __init__(self, macro_value: 'DmxValue' = None, dmx_channel: 'NodeLink' = None, *args, **kwargs):
+        self.value = macro_value
         self.dmx_channel = dmx_channel
         super().__init__(*args, **kwargs)
 
     def _read_xml(self, xml_node: 'Element'):
-        self.value = pygdtf.type.DmxValue(xml_node.get('Value'))
-        self.dmx_channel = pygdtf.type.NodeLink('DMXChannelCollect', xml_node.get('DMXChannel'))
+        self.value = DmxValue(xml_node.attrib.get('Value'))
+        self.dmx_channel = NodeLink('DMXChannelCollect', xml_node.attrib.get('DMXChannel'))
 
 
 class MacroVisualStep(BaseNode):
@@ -747,22 +776,22 @@ class MacroVisualStep(BaseNode):
         super().__init__(*args, **kwargs)
 
     def _read_xml(self, xml_node: 'Element'):
-        self.duration = int(xml_node.get('Duration', default=1))
-        self.fade = float(xml_node.get('Fade', default=0.0))
-        self.delay = float(xml_node.get('Delay', default=0.0))
+        self.duration = int(xml_node.attrib.get('Duration', 1))
+        self.fade = float(xml_node.attrib.get('Fade', 0.0))
+        self.delay = float(xml_node.attrib.get('Delay', 0.0))
         self.visual_values = [MacroVisualValue(xml_node=i) for i in xml_node.findall('MacroVisualValue')]
 
 
 class MacroVisualValue(BaseNode):
 
-    def __init__(self, value: 'DmxValue' = None, channel_function: 'NodeLink' = None, *args, **kwargs):
-        self.value = value
+    def __init__(self, macro_value: 'DmxValue' = None, channel_function: 'NodeLink' = None, *args, **kwargs):
+        self.value = macro_value
         self.channel_function = channel_function
         super().__init__(*args, **kwargs)
 
     def _read_xml(self, xml_node: 'Element'):
-        self.value = pygdtf.type.DmxValue(xml_node.get('Value'))
-        self.channel_function = pygdtf.type.NodeLink('DMXChannelCollect', xml_node.get('ChannelFunction'))
+        self.value = DmxValue(xml_node.attrib.get('Value'))
+        self.channel_function = NodeLink('DMXChannelCollect', xml_node.attrib.get('ChannelFunction'))
 
 
 class Revision(BaseNode):
@@ -774,6 +803,6 @@ class Revision(BaseNode):
         super().__init__(*args, **kwargs)
 
     def _read_xml(self, xml_node: 'Element'):
-        self.text = xml_node.get('Text')
-        self.date = xml_node.get('Date')
-        self.user_id = int(xml_node.get('UserID'))
+        self.text = xml_node.attrib.get('Text')
+        self.date = xml_node.attrib.get('Date')
+        self.user_id = int(xml_node.attrib.get('UserID'))
